pub struct PedersenCommitter<C: PrimeOrderCurve> {
pub generators: Vec<C>,
pub blinding_generator: C,
pub int_abs_val_bitwidth: usize,
generator_doublings: Vec<Vec<C>>,
}Expand description
For committing to vectors of integers and scalars using the Pedersen commitment scheme.
Fields
generators: Vec<C>vector of “g” generators, i.e. the generators that are exponentiated by the message elements themselves (length > 0) The first N generators are used to commit to a vector of N values. The last generator is used for scalar commitments.
blinding_generator: Cthe “h” generator which is exponentiated by the blinding factor
int_abs_val_bitwidth: usizeThe bitwidth of the absolute values of the integers that can be committed to using integral vector commit methods (has no bearing on scalar_commit and vector_commit).
generator_doublings: Vec<Vec<C>>Implementations
Sourceimpl<C: PrimeOrderCurve> PedersenCommitter<C>
impl<C: PrimeOrderCurve> PedersenCommitter<C>
const DEFAULT_INT_ABS_VAL_BITWIDTH: usize = 8
Sourcepub fn new(
num_generators: usize,
public_string: &str,
int_abs_val_bitwidth: Option<usize>,
) -> Self
pub fn new( num_generators: usize, public_string: &str, int_abs_val_bitwidth: Option<usize>, ) -> Self
Creates a new PedersenCommitter with random generators. See also PedersenCommitter. Generators are sampled using the public string and the Shake256 hash function. Post: self.generators.len() == num_generators
Sourcepub fn scalar_commit_generator(&self) -> C
pub fn scalar_commit_generator(&self) -> C
Return the generator used for scalar commitments.
Sourcefn sample_generators(num_generators: usize, public_string: &str) -> Vec<C>
fn sample_generators(num_generators: usize, public_string: &str) -> Vec<C>
Sample generators using the public string and the Shake256 hash function. Pre: public_string.len() >= 32 Post: result.len() == num_generators TODO: make seekable (block cipher)
Sourcepub fn new_with_generators(
generators: Vec<C>,
blinding_generator: C,
int_abs_val_bitwidth: Option<usize>,
) -> Self
pub fn new_with_generators( generators: Vec<C>, blinding_generator: C, int_abs_val_bitwidth: Option<usize>, ) -> Self
Create a new PedersenCommitter with the provided generators.
See PedersenCommitter.
DEFAULT_INT_ABS_VAL_BITWIDTH is used for int_abs_val_bitwidth if None is provided.
Sourcepub fn unsigned_integer_vector_commit<T: Unsigned + Zero + ToBytes>(
&self,
message: &[T],
blinding: &C::Scalar,
) -> C
pub fn unsigned_integer_vector_commit<T: Unsigned + Zero + ToBytes>( &self, message: &[T], blinding: &C::Scalar, ) -> C
An optimized version of Pedersen vector commit when the message is comprised of values that fit within 128 bits.
Sourcepub fn u8_vector_commit(&self, message: &[u8], blinding: &C::Scalar) -> C
pub fn u8_vector_commit(&self, message: &[u8], blinding: &C::Scalar) -> C
Commits to the vector of u8s using the specified blinding factor. Uses the precomputed generator powers and the binary decomposition. Convient wrapper of integer_vector_commit. Pre: self.int_abs_val_bitwidth >= 8. Post: same result as vector_commit, assuming uints are smaller than scalar field order.
Sourcepub fn i8_vector_commit(&self, message: &[i8], blinding: &C::Scalar) -> C
pub fn i8_vector_commit(&self, message: &[i8], blinding: &C::Scalar) -> C
Commits to the vector of i8s using the specified blinding factor. Uses the precomputed generator powers and the binary decomposition. Convient wrapper of integer_vector_commit. Pre: self.int_abs_val_bitwidth >= 8. Post: same result as vector_commit, assuming ints are smaller than scalar field order.
Sourcepub fn integer_vector_commit<T: PrimInt>(
&self,
message: &[T],
message_is_negative_bits: &[bool],
blinding: &C::Scalar,
) -> C
pub fn integer_vector_commit<T: PrimInt>( &self, message: &[T], message_is_negative_bits: &[bool], blinding: &C::Scalar, ) -> C
Commits to the vector of integers using the specified blinding factor. Integers are provided as a vector of UNSIGNED ints and a vector of bits indicating whether the integer is negative. Pre: values in message are non-negative. Pre: values have unsigned binary expressions using at most (self.highest_generator_power + 1) bits. Pre: message.len() <= self.message_generators.len()
Sourcepub fn vector_commit(&self, message: &[C::Scalar], blinding: &C::Scalar) -> C
pub fn vector_commit(&self, message: &[C::Scalar], blinding: &C::Scalar) -> C
Commit to the provided vector using the specified blinding factor. The first message.len() generators are used to commit to the message. Note that self.int_abs_val_bitwidth is not relevant here. Pre: message.len() <= self.message_generators.len()
Sourcepub fn committed_vector(
&self,
message: &[C::Scalar],
blinding: &C::Scalar,
) -> CommittedVector<C>
pub fn committed_vector( &self, message: &[C::Scalar], blinding: &C::Scalar, ) -> CommittedVector<C>
Convenience wrapper of [self.vector_commit] that returns the commitment wrapped in a CommittedVector.
Sourcepub fn scalar_commit(&self, message: &C::Scalar, blinding: &C::Scalar) -> C
pub fn scalar_commit(&self, message: &C::Scalar, blinding: &C::Scalar) -> C
Commit to the provided scalar using the specified blinding factor. Note that self.int_abs_val_bitwidth is not relevant here. Pre: self.message_generators.len() >= 1
Sourcepub fn committed_scalar(
&self,
message: &C::Scalar,
blinding: &C::Scalar,
) -> CommittedScalar<C>
pub fn committed_scalar( &self, message: &C::Scalar, blinding: &C::Scalar, ) -> CommittedScalar<C>
Convenience wrapper of [self.scalar_commit] that returns the commitment wrapped in a CommittedScalar.